@@ -54,11 +54,11 @@ static char *filenames[]={
     "flash_flood>statement", "flood>warning", "flood>coastal",
     "flood>statement", "severe_weather_stmt", "special_weather_stmt",
     "thunderstorm", "special_marine", "urgent_weather_message", "non_precip",
-    "fire_weather", "lake_shore", NULL
+    "fire_weather", "lake_shore", "tsunami", "tsunami_seismic_msg", NULL
 };
 static char **reqs[sizeof(filenames)/sizeof(*filenames)-1][2];
 
-unsigned long current_warnings;
+unsigned long current_warnings, any_warnings;
 static unsigned long *zone_current_warnings;
 
 /* Regular Expressions */
@@ -114,6 +114,7 @@ void init_warnings(void){
 
     /* Setup misc vars */
     current_warnings=0;
+    any_warnings=0;
     ovecsize=0;
     compile(expires, "Expires:(\\d+)(\\d\\d)(\\d\\d)(\\d\\d)(\\d\\d);");
     ovecsize=(ovecsize+1)*3;
@@ -154,6 +155,7 @@ static void warning_callback(char *filename, void *v){
            && check_warning(filename)
            && diff(filename, warning_filename1)){
             current_warnings|=1<<d->warning;
+            any_warnings|=1<<d->warning;
             zone_current_warnings[d->zone]|=1<<d->warning;
             rename(filename, warning_filename1);
         } else {
@@ -181,6 +183,7 @@ void update_warnings(int force){
     struct stat statbuf;
     int i, z;
     struct callback_data *d;
+    unsigned long cur_warnings = 0;
 
     if(warning_filename1==NULL) return;
 //    t=time(NULL)/60;
@@ -196,11 +199,11 @@ void update_warnings(int force){
                 if(!S_ISREG(statbuf.st_mode) || statbuf.st_size==0
                    || !check_warning(warning_filename1)){
                     unlink(warning_filename1);
-                    current_warnings&=~(1<<i);
                     zone_current_warnings[z]&=~(1<<i);
+                } else {
+                    cur_warnings|=1<<i;
                 }
             } else {
-                current_warnings&=~(1<<i);
                 zone_current_warnings[z]&=~(1<<i);
             }
 
@@ -211,6 +214,8 @@ void update_warnings(int force){
             download_file(warning_filename1, reqs[i][0][z], reqs[i][1][z], DOWNLOAD_NO_404, warning_callback, d);
         }
     }
+    current_warnings&=cur_warnings;
+    any_warnings&=cur_warnings;
 }
 
 
@@ -262,6 +267,7 @@ void output_warnings(int all){
     pid_t pid;
     int pipefd[2];
 
+    if(any_warnings==0) return;
     if(!all && current_warnings==0) return;
 
     if(pipe(pipefd)) die("output_warnings pipe creation");
@@ -284,7 +290,7 @@ void output_warnings(int all){
     /* PARENT writes warnings to the pipe and returns */
     close(pipefd[0]);
     for(z=0; warning_zones[z]!=NULL; z++){
-        if(!zone_current_warnings[z]) continue;
+        if(!all && !zone_current_warnings[z]) continue;
         for(i=0; filenames[i]!=NULL; i++){
             if(!all && !(zone_current_warnings[z]&(1<<i))) continue;
 
